@@ -45,6 +45,20 @@ static enum cross_act check_surrounding (Coord, map_val);
 static void set_modes ();
 static int min (int, int);
 
+extern void
+init_map ()
+{
+  game_map.begin_turn = 1;
+  game_map.players_left = game_map.players;
+  game_map.n_active_players = game_map.players;
+  game_map.n_cross=0;
+  game_map.bit_cross=0;
+
+  game_map.size = game_map.width * game_map.height;
+  game_map.map = (map_val *) my_malloc (game_map.size * sizeof (map_val));
+  game_map.capital = (Coord *) my_malloc (game_map.players * sizeof (Coord));
+}
+
 static int
 noslashes (str)
      char *str;
@@ -125,7 +139,6 @@ load_map (map_dir)
 	die ("can't open file %s", map_dir);
     }
 
-  /* TODO: Should there be more variables in map file?  */
   game_map.width=read_number (file, &status);
   game_map.height=read_number (file, &status);
   game_map.warped=read_number (file, &status);
@@ -138,15 +151,7 @@ load_map (map_dir)
   if (status != 0)
     die ("invalid map file header: %s", map_dir);
 
-  game_map.begin_turn = 1;
-  game_map.players_left = game_map.players;
-  game_map.n_active_players = game_map.players;
-  game_map.n_cross=0;
-  game_map.bit_cross=0;
-
-  game_map.size = game_map.width * game_map.height;
-  game_map.map = (map_val *) my_malloc (game_map.size * sizeof (map_val));
-  game_map.capital = (Coord *) my_malloc (game_map.players * sizeof (Coord));
+  init_map ();
 
   for (i = 0; i < game_map.size; i++)
     {
@@ -202,6 +207,61 @@ load_map (map_dir)
   game_map.sel_mode = game_map.def_mode;
 }
 
+extern void
+save_map (file_name)
+     const char *file_name;
+{
+  register int x, y, index;
+  FILE *file;
+  file = fopen (file_name, "w");
+  if (!file)
+    {
+      perror ("motti: can't open map save file");
+      exit (1);
+    }
+
+  /* Uh, I'm lazy.  If the write fails here, it will fail also later...  */
+  fprintf (file, "width %d\n", game_map.width);
+  fprintf (file, "height %d\n", game_map.height);
+  fprintf (file, "warped %d\n", game_map.warped);
+  fprintf (file, "players %d\n", game_map.players);
+  fprintf (file, "active_players %d\n", game_map.active_players);
+  fprintf (file, "remaining_players %d\n", game_map.remaining_players);
+  fprintf (file, "turn %d\n", game_map.turn);
+  fprintf (file, "n_att %d\n", game_map.n_att);
+
+  for (y = 0, index = 0; y < game_map.height; y++)
+    {
+      int error;
+      for (x = 0; x < game_map.width; x++, index++)
+	{
+	  if (game_map.map[index] == SEA_VAL)
+	    error = fwrite ("  ", 1, 2, file);
+	  else
+	    {
+	      error = fputc ((game_map.map[index] & MASK_PLAYER)
+			     + ((game_map.map[index] & MASK_OCCUPIED)
+				? 'A'-1 : 'a'-1), file);
+	      if (game_map.map[index] & MASK_CAPITAL)
+		error = fputc ((game_map.map[index] & MASK_CROSS)
+			       ? 'X' : '.', file);
+	      else
+		error = fputc ((game_map.map[index] & MASK_CROSS)
+			       ? 'x' : ' ', file);
+	    }
+	  if (error == EOF)
+	    goto fail;
+	}
+      error = fputc ('\n', file);
+      if (error == EOF)
+	{
+	fail:
+	  die ("map write failed");
+	}
+    }
+  fclose (file);
+}
+
 extern void
 get_map_real_coord (loc)
      Coord *loc;
