@@ -24,7 +24,8 @@ enum connect_type {
   CONNECTED,
   BORDER,
   PASSED,
-  NOT_ACCOUNTED
+  NOT_ACCOUNTED,
+  IGNORED
 };
 
 #include <stdlib.h>
@@ -70,16 +71,18 @@ defeat_players (pl_mask, act)
   register int i, j;
   int have_defeats = 0, need_turnchange = 0;
 
+  act->type = 0;
+  act->count = 0;
+  if (!pl_mask)
+    return;
+  for (j = 0; j < game_map.size; j++)
+    connect[j] = CONNECTED;
+
   for (i = 1; i <= game_map.players; i++)
     if (1<<(i-1) & pl_mask)
       {
 	int mask = ~(1<<(i-1));
-	if (!have_defeats)
-	  {
-	    for (j = 0; j < game_map.size; j++)
-	      connect[j] = CONNECTED;
-	    have_defeats = 1;
-	  }
+
 	if (game_map.turn == i)
 	  need_turnchange = 1;
 
@@ -88,30 +91,27 @@ defeat_players (pl_mask, act)
 	game_map.n_active_players--;
 	game_map.players_left--;
 	for (j = 0; j < game_map.size; j++)
-	  if (game_map.map[j] == SEA_VAL
-	      || (game_map.map[j] & MASK_PLAYER) == i)
+	  if ((game_map.map[j] & MASK_PLAYER) == i)
 	    connect[j] = UNCONNECTED;
       }
+  for (i = 0; i < game_map.size; i++)
+    if (game_map.map[i] == SEA_VAL)
+      connect[i] = IGNORED;
 
-  act->type = 0;
-  act->count = 0;
-  if (have_defeats)
+  if (game_map.remaining_players > 1)
     {
-      if (game_map.remaining_players > 1)
+      player = 0;
+      while (search_encircled ());
+      if (need_turnchange)
 	{
-	  player = 0;
-	  while (search_encircled ());
-	  if (need_turnchange)
-	    {
-	      if (new_turn ())
-		act->type = EVENT_NEWTURN|EVENT_UNOCCUPY|EVENT_DEFEAT;
-	      else
-		act->type = EVENT_NEWTURN|EVENT_DEFEAT;
-	      return;
-	    }
+	  if (new_turn ())
+	    act->type = EVENT_NEWTURN|EVENT_UNOCCUPY|EVENT_DEFEAT;
+	  else
+	    act->type = EVENT_NEWTURN|EVENT_DEFEAT;
+	  return;
 	}
-      act->type = EVENT_DEFEAT;
     }
+  act->type = EVENT_DEFEAT;
 }
 
 /* Mark all other players' occupied cells, which are next to any cells passed
@@ -128,10 +128,7 @@ borders (loc)
       /* Ok from the first pass.  */
       if (connect[ref] == PASSED
 	  /* Not yet passed on the second pass.  */
-	  || (connect[ref] != NOT_ACCOUNTED
-	      && connect[ref] != CONNECTED
-	      /* Not a sea.  */
-	      && game_map.map[ref] != SEA_VAL
+	  || ((connect[ref] == UNCONNECTED || connect[ref] == BORDER)
 	      /* Other player's cell.  */
 	      && (game_map.map[ref] & MASK_PLAYER) != player))
 	return SUCCESSFUL;
@@ -163,8 +160,6 @@ blocked (loc)
       register int ref = parse_coord (loc);
       /* Test if this cell is yet unconnected.  */
       if (connect[ref] == UNCONNECTED
-	  /* Not a sea.  */
-	  && game_map.map[ref] != SEA_VAL
 	  /* Not an other player's occupied cell.  */
 	  && !((game_map.map[ref] & MASK_PLAYER) != player
 	       && (game_map.map[ref] & MASK_OCCUPIED)))
@@ -192,7 +187,7 @@ not_connected (loc)
     {
       register int ref;
       ref = parse_coord (loc);
-      if (game_map.map[ref] != SEA_VAL && connect[ref] == UNCONNECTED)
+      if (connect[ref] == UNCONNECTED)
 	return SUCCESSFUL;
     }
   return UNSUCCESSFUL;
@@ -225,7 +220,7 @@ mark_border (loc)
 	      connect[loc_ref] = BORDER;
 	      if ((game_map.map[loc_ref] & MASK_PLAYER) == player
 		  /* For general defeats.  */
-		  || (player == 0 && game_map.map[loc_ref] != SEA_VAL))
+		  || player == 0)
 		invade (loc);	/* Encircled player's cell.  */
 	      return UNSUCCESSFUL;
 	    }
@@ -233,7 +228,7 @@ mark_border (loc)
     }
   connect[loc_ref] = PASSED;
   /* Return successful if there are more enemy cells to invade.  */
-  if ((game_map.map[loc_ref] & MASK_PLAYER) == player)
+  if ((game_map.map[loc_ref] & MASK_PLAYER) == player || player == 0)
     return SUCCESSFUL;
   else
     return UNSUCCESSFUL;
@@ -246,8 +241,7 @@ search_encircled ()
   for (i = 0; i < game_map.size; i++)
     {
       if (connect[i] == UNCONNECTED
-	  && ((game_map.map[i] & MASK_PLAYER) == player
-	      || (player == 0 && game_map.map[i] != SEA_VAL)))
+	  && ((game_map.map[i] & MASK_PLAYER) == player || player == 0))
 	{
 	  /* Run only if necessary, since this is a quite expensive function.
 	     First pass was in function fill_check.  */
@@ -343,7 +337,7 @@ count_connects (loc, pattern, array)
 	  map_val val;
 	  ref = parse_real_coord (this_loc);
 	  val = game_map.map[ref] & MASK_PLAYER;
-	  if (val != SEA_VAL && connect[ref] == CONNECTED)
+	  if (connect[ref] == CONNECTED)
 	    {
 	      if (game_map.map[ref] & MASK_OCCUPIED)
 		array[val - 1] += 3;
@@ -412,10 +406,18 @@ destroy ()
   game_map.active_players &= ~(1<<(player - 1));
   game_map.remaining_players &= ~(1<<(player - 1));
   game_map.n_active_players--;
+
   /* Mark all cells of the defeated player and seas to be unconnected.  */
   for (i = 0; i < game_map.size; i++)
-    connect[i] = game_map.map[i] == SEA_VAL
-      || (game_map.map[i] & MASK_PLAYER) == player ? UNCONNECTED : CONNECTED;
+    {
+      if (game_map.map[i] == SEA_VAL)
+	connect[i] = IGNORED;
+      else if ((game_map.map[i] & MASK_PLAYER) == player)
+	connect[i] = UNCONNECTED;
+      else
+	connect[i] = CONNECTED;
+    }
+
   while (search_encircled ());
 }
 
@@ -457,7 +459,7 @@ occupy (act, cross_n)
 	{
 	  int j;
 	  for (j = 0; j < game_map.size; j++)
-	    connect[j] = UNCONNECTED;
+	    connect[j] = game_map.map[j] == SEA_VAL ? IGNORED : UNCONNECTED;
 	}
       player = i;
       /* FIXME: a quick check to avoid the expensive fill_check (),
