@@ -33,51 +33,16 @@ extern void
 die (const char *msg, ...)
 {
   va_list args;
-  char *fullmsg;
+  char fullmsg[256];	/* Should be enough for every message.  */
   int len;
   va_start (args, msg);
 
-  len = strlen (msg)+1;
-  fullmsg = my_malloc (len * sizeof (char));
-
   sprintf (fullmsg, "motti: %s\n", msg);
   vfprintf (stderr, fullmsg, args);
   va_end (args);
-  free (fullmsg);
   exit (1);
 }
 
-extern char *
-alloc_err_str (err_no, file_name)
-     enum motti_errors err_no;
-     const char *file_name;
-{
-  char *str;
-  const char *msg[] =
-  {
-    "can't find map file: %s",
-    "invalid map file: %s",
-    "can't open display: %s\ntry motti -n for text mode",
-    "can't load font: %s",
-    /* Fix me.  */
-    "malloc error",
-    "can't find color: %s",
-    "can't allocate a color: %s",
-    "invalid mouse action: %s"
-  };
-#if 0
-  const int msg_len[] =
-  {
-    22, 19, 48, 18, 13, 19, 25, 23
-  };
-#endif
-  str = (char *) my_malloc ((sizeof (msg[err_no]) + file_name == NULL
-			     ? strlen (file_name) : 0) * sizeof
-			    (char));
-  sprintf (str, msg[err_no], file_name);
-  return str;
-}
-
 extern void *
 my_malloc (size)
      size_t size;
@@ -86,8 +51,7 @@ my_malloc (size)
   ptr = malloc(size);
   if (ptr == NULL && size != 0)
     {
-      /* Function die uses my_malloc, so we can't use it here.  */
-      fprintf (stderr, "motti: malloc error\n");
+      die ("malloc error");
       exit (1);
     }
   return ptr;
