@@ -35,17 +35,16 @@
 #include "xwin.h"
 #include "xinit.h"
 #include "map.h"
-#include "thread.h"
 
-static void update_borders (struct x_info_struct *);
+static void x_update_borders (struct x_info_struct *);
 static int set_fill_color (struct x_info_struct *, map_val);
-static void draw_map_function (struct x_info_struct *, Coord, Coord);
 
 static void
-update_borders (x_info)
+x_update_borders (x_info)
      struct x_info_struct *x_info;
 {
-  if (x_info->last_sel != game_map.sel_mode)
+  if (x_info->last_sel != game_map.sel_mode
+      || (x_info->last_sel && !x_info->active))
     {
       switch (x_info->last_sel)
 	{
@@ -59,6 +58,11 @@ update_borders (x_info)
 	  XSetWindowBorderWidth (x_info->display, x_info->but_win[GUE], 1);
 	  break;
 	}
+      if (!x_info->active)
+	{
+	  x_info->last_sel = 0;
+	  return;
+	}
       switch (game_map.sel_mode)
 	{
 	case MODE_ATT:
@@ -79,33 +83,34 @@ update_borders (x_info)
 }
 
 extern void
-update_buttons (x_info, uses_def_mode, last_modes)
+x_update_buttons (x_info)
      struct x_info_struct *x_info;
-     const int uses_def_mode;
-     char *last_modes;
 {
   char update;
-  update = need_update (last_modes);
-  if (uses_def_mode && x_info)
-    update_borders (x_info);
+  update = need_update (&x_info->last_modes);
+  if (x_info->uses_def_mode)
+    x_update_borders (x_info);
 
   if (update & MODE_ATT)
     {
       if (game_map.modes & MODE_ATT)
-        XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[ATT],
-				    x_info->att_pix[5 - game_map.n_att]);
+	{
+	  if (x_info->active)
+	    XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[ATT],
+					x_info->att_pix[5 - game_map.n_att]);
+	  else
+	    XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[ATT],
+					x_info->d_att_pix[6 - game_map.n_att]);
+	}
       else
 	XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[ATT],
 				    x_info->d_att_pix[0]);
       XClearWindow (x_info->display, x_info->but_win[ATT]);
-#ifdef HAVE_LIBPTHREAD
-      expose_att_but (x_info->conn_num);
-#endif
     }
 
   if (update & MODE_DEF)
     {
-      if (game_map.modes & MODE_DEF)
+      if ((game_map.modes & MODE_DEF) && x_info->active)
 	XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[DEF],
 				    x_info->def_pix);
       else
@@ -116,7 +121,7 @@ update_buttons (x_info, uses_def_mode, last_modes)
 
   if (update & MODE_GUE)
     {
-      if (game_map.modes & MODE_GUE)
+      if ((game_map.modes & MODE_GUE) && x_info->active)
 	XSetWindowBackgroundPixmap (x_info->display, x_info->but_win[GUE],
 				    x_info->gue_pix);
       else
@@ -127,7 +132,7 @@ update_buttons (x_info, uses_def_mode, last_modes)
 }
 
 extern void
-draw_turn_win (x_info)
+x_draw_turn_win (x_info)
      struct x_info_struct *x_info;
 {
   unsigned char symbol_n;
@@ -170,8 +175,8 @@ set_fill_color (x_info, status)
   return 1;
 }
   
-static void
-draw_map_function (x_info, loc, dim)
+extern void
+x_draw_map (x_info, loc, dim)
      struct x_info_struct *x_info;
      Coord loc, dim;
 {
@@ -233,62 +238,19 @@ draw_map_function (x_info, loc, dim)
 }
 
 extern void
-draw_map (x_info, x, y, width, height)
+translate_box (x_info, x, y, width, height, loc, dim)
      struct x_info_struct *x_info;
      int x, y, width, height;
+     Coord *loc, *dim;
 {
-  Coord loc, dim;
-
-  loc.x = x / x_info->db_tab.map_square_size;
-  loc.y = y / x_info->db_tab.map_square_size;
-  dim.x = (width % x_info->db_tab.map_square_size ? width /
+  loc->x = x / x_info->db_tab.map_square_size;
+  loc->y = y / x_info->db_tab.map_square_size;
+  dim->x = (width % x_info->db_tab.map_square_size ? width /
 	   x_info->db_tab.map_square_size + 1 :
-	   width / x_info->db_tab.map_square_size)+loc.x; 
-  dim.y = (height % x_info->db_tab.map_square_size ? height /
+	   width / x_info->db_tab.map_square_size)+loc->x; 
+  dim->y = (height % x_info->db_tab.map_square_size ? height /
 	   x_info->db_tab.map_square_size + 1 :
-	   height / x_info->db_tab.map_square_size)+loc.y;
-
-  draw_map_function (x_info, loc, dim);
-}
-
-extern void
-draw_effects (x_info, map_events)
-     struct x_info_struct *x_info;
-     Action map_events;
-{
-  draw_turn_win (x_info);
-#ifdef HAVE_LIBPTHREAD
-  expose_turnwin (x_info->conn_num);
-#endif
-
-  if ((map_events.type & (EVENT_ENCIRCLEMENT | EVENT_UNOCCUPY |
-			  EVENT_DEFEAT | EVENT_REFRESH)) ||
-      ((map_events.type & EVENT_ATT) &&
-      (map_events.type & EVENT_NEWTURN)))
-    {
-      Coord top_left = {0, 0}, down_right;
-
-      down_right.x = game_map.width;
-      down_right.y = game_map.height;
-#ifdef HAVE_LIBPTHREAD
-      expose_map (x_info->conn_num, top_left.x, top_left.y, down_right.x,
-		  down_right.y);
-#endif
-      draw_map_function (x_info, top_left, down_right);
-    }
-  else
-    {
-      register int i;
-      for (i = 0; i < map_events.count; i++)
-	{
-#ifdef HAVE_LIBPTHREAD
-	  expose_map (x_info->conn_num, map_events.loc[i].x,
-		      map_events.loc[i].y, 0, 0);
-#endif
-	  draw_map_function (x_info, map_events.loc[i], map_events.loc[i]);
-	}
-    }
-  free (map_events.loc);
+	   height / x_info->db_tab.map_square_size)+loc->y;
 }
 
 extern Coord
@@ -303,7 +265,7 @@ win_pos2map_coord (x_info, x, y)
 }
 
 extern void
-create_end_win (x_info)
+x_create_end_win (x_info)
      struct x_info_struct *x_info;
 {
   int null;
@@ -343,10 +305,13 @@ create_end_win (x_info)
 			 charstr.width, charstr.ascent + charstr.descent, 1,
 			 x_info->misc_col[end_bor_col].pixel,
 			 x_info->misc_col[end_bg_col].pixel);
+  XSelectInput (x_info->display, x_info->endgame_win,
+		ExposureMask | ButtonReleaseMask);
+  XMapWindow (x_info->display, x_info->endgame_win);
 }
 
 extern void
-draw_end_win (x_info)
+x_draw_end_win (x_info)
      struct x_info_struct *x_info;
 {
   XDrawImageString (x_info->display, x_info->endgame_win,
